/*
 *  This file is part of the caQtDM Framework, developed at the Paul Scherrer Institut,
 *  Villigen, Switzerland
 *
 *  The caQtDM Framework is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The caQtDM Framework is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with the caQtDM Framework.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (c) 2010 - 2014
 *
 *  Author:
 *    Anton Mezger
 *  Contact details:
 *    anton.mezger@psi.ch
 */

#include "catextentry.h"
#include <QApplication>
#include <QMouseEvent>
// Zai added
#include <cacalc.h>
#include <alarmdefs.h>
#include <QInputDialog>
#include <QFileDialog>

caTextEntry::caTextEntry(QWidget *parent) : caLineEdit(parent)
{
  // this dis not really worked on ios, while the events had another order
  //connect(this, SIGNAL(returnPressed()), this, SLOT(dataInput()));
    clearFocus();
    setAccessW(true);
    installEventFilter(this);
    newFocusPolicy(Qt::ClickFocus);
    this->setAcceptDrops(false);
    setFromTextEntry();

    setElevation(on_top);
}

// routine not used any more
void caTextEntry::dataInput()
{
    //printf("dataInput %s\n", qasc(text()));
    emit TextEntryChanged(text());
}

void caTextEntry::setAccessW(bool access)
{
    _AccessW = access;
}

void caTextEntry::updateText(const QString &txt)
{
    //printf("text written by CS %s\n", qasc(txt));
    startText = txt;
}

bool caTextEntry::eventFilter(QObject *obj, QEvent *event)
{
    // repeat enter or return key are not really wanted
	if (event->type() == QEvent::KeyPress){
		QKeyEvent *ev = static_cast<QKeyEvent *>(event);
		if (ev != (QKeyEvent *)0) {
			if (ev->key() == Qt::Key_Return || ev->key() == Qt::Key_Enter) {
				if (ev->isAutoRepeat()) {
                    //printf("keyPressEvent ignore\n");
                    event->ignore();
				}
				else {
					event->accept();
                    //printf("keyPressEvent accept, set text to %s entered=%s ?\n", qasc(startText), qasc(text()));
                    // Zai request data directly
                    caTextEntry::FormatType fType = getFormatType();
                    if(getAccessW()){
                        CaQtDM_Lib_Interface::formatsType fTypeNew;
                        if(fType == caTextEntry::octal) fTypeNew = CaQtDM_Lib_Interface::octal;
                        else if(fType == caTextEntry::hexadecimal) fTypeNew = CaQtDM_Lib_Interface::hexadecimal;
                        else if(fType == caTextEntry::string) fTypeNew = CaQtDM_Lib_Interface::string;
                        else fTypeNew = CaQtDM_Lib_Interface::decimal;
                        if (caDataInterface){
                            caDataInterface->TreatRequestedValue(getPV(), text(), fTypeNew, this);
                        }
                    }
// Zai                    emit TextEntryChanged(text().toAscii().constData());
                }
			}
		}
	}
    // treat mouse enter and leave as well as focus out
    if (event->type() == QEvent::Enter) {
        if(!_AccessW) {
            QApplication::setOverrideCursor(QCursor(Qt::ForbiddenCursor));
            setReadOnly(true);
        } else {
            QApplication::restoreOverrideCursor();
        }
        this->activateWindow();  // I added this for ios while I could not get the focus
    } else if(event->type() == QEvent::Leave) {
        QApplication::restoreOverrideCursor();
        setReadOnly(false);
        clearFocus();
    } else if(event->type() == QEvent::FocusOut) {
        //printf("lost focus, set text to %s\n", qasc(startText));
        forceText(startText);
    } else if (event->type() == QEvent::FocusIn) {
        //printf("focus in\n");
    }
    return QObject::eventFilter(obj, event);
}

// Zai - caWidgetInterface implementation
void caTextEntry::caDataUpdate(const QString& units, const QString& String, const knobData& data){
    //qDebug() << "we have a linedit or textentry" << widget << data.edata.rvalue <<  data.edata.ivalue;
    QColor bg = property("BColor").value<QColor>();
    QColor fg = property("FColor").value<QColor>();

    if(data.edata.connected) {
        // enum string
        if(data.edata.fieldtype == caENUM || data.edata.fieldtype == caSTRING || data.edata.fieldtype == caCHAR) {
            QStringList list;
            setValueType(false);
            int colorMode = getColorMode();
            if(colorMode == caLineEdit::Static || colorMode == caLineEdit::Default) { // done at initialisation
                if(!property("Connect").value<bool>()) {                    // but was disconnected before
                    setAlarmColors(data.edata.severity, (double) data.edata.ivalue, bg, fg);
                    setProperty("Connect", true);
                }
            } else {
                setAlarmColors(data.edata.severity, (double) data.edata.ivalue, bg, fg);
            }
            list = String.split(";");
            //qDebug() << widget << String << list << data.pv << (int) data.edata.ivalue << data.edata.valueCount;

            if((data.edata.fieldtype == caENUM)  && (list.count() == 0)) {
                QString str= QString::number((int) data.edata.ivalue);
                setText(str);
            } else if((data.edata.fieldtype == caENUM)  && ((int) data.edata.ivalue < list.count() ) && (list.count() > 0)) {
                if(list.at((int) data.edata.ivalue).trimmed().size() == 0)  {  // string seems to emty, give value
                    QString str= QString::number((int) data.edata.ivalue);
                    setText(str);
                } else {
                    setText(list.at((int) data.edata.ivalue));
                }
            } else if((data.edata.fieldtype == caENUM)  && ((int) data.edata.ivalue >= list.count()) && (list.count() > 0)) {
                QString str= QString::number((int) data.edata.ivalue);
                setText(str);
            } else if (data.edata.fieldtype == caENUM) {
                setText("???");
            } else {
                if(data.edata.valueCount == 1) {
                    setText(String);
                } else if(list.count() > 0) {
                    setText(list.at(0));
                }
            }
            setCursorPosition(0);
            // access control for textentry
            if (caTextEntry *widget = qobject_cast<caTextEntry *>(this)) {
                widget->setAccessW(data.edata.accessW);
            }
            // double
        } else {
            int colorMode = getColorMode();
            int precMode = getPrecisionMode();
            setValueType(true);

            if(colorMode == caLineEdit::Static || colorMode == caLineEdit::Default) { // done at initialisation
                if(!property("Connect").value<bool>()) {                      // but was disconnected before
                    setAlarmColors(data.edata.severity, data.edata.rvalue, bg, fg);
                    setProperty("Connect", true);
                }
            } else {
                setAlarmColors(data.edata.severity, data.edata.rvalue, bg, fg);
            }

            if((precMode != caLineEdit::User) && (data.edata.initialize)) {
                setFormat(data.edata.precision);
            }

            setValue(data.edata.rvalue, units);
            // access control for textentry
            if (caTextEntry *widget = qobject_cast<caTextEntry *>(this)) {
                widget->setAccessW(data.edata.accessW);
            }
        }

    } else {
        setValueType(false);
        setText("");
        setAlarmColors(NOTCONNECTED, 0.0, bg, fg);
        setProperty("Connect", false);
    }
    if (caTextEntry *widget = qobject_cast<caTextEntry *>(this)) {
        widget->updateText(widget->text());
    }
}

void caTextEntry::caActivate(CaQtDM_Lib_Interface* interface, QMap<QString, QString> map, knobData* kData, int* specData, QWidget* parent){
    qDebug() << "create caTextEntry";
    setProperty("ObjectType", caTextEntry_Widget);

    if(getPV().size() > 0) {
        setEnabled(true);
        QString pv;
        interface->addMonitor(parent, kData, getPV(), (QWidget*)this, specData, map, &pv);
        setPV(pv);
//            connect(this, SIGNAL(TextEntryChanged(const QString&)), this,
//                    SLOT(textEntryChanged(const QString&)));
    }
    // default format, format from ui file will be used normally except for channel precision
    setFormat(1);
    clearFocus();
    setProperty("Taken", true);
    caDataInterface = interface;
}

void caTextEntry::createContextMenu(QMenu& menu){
    // construct info for the pv we are pointing at
    menu.addAction("Get Info");
    if(getAccessW()) {
        knobData *kPtr = caDataInterface->GetMutexKnobDataPV(this, getPV().trimmed());  // use pointer for getting all necessary information
        if((kPtr != (knobData *) 0) && (getPV().trimmed().length() > 0)) {
            menu.addAction("Input Dialog");
            if((kPtr->edata.fieldtype == caSTRING) || (kPtr->edata.fieldtype == caCHAR)) {
                menu.addAction("File Dialog");
            }
        }
    }
    menu.addAction("Change Limits/Precision");
    QAction* selectedItem = menu.exec(QCursor::pos());
    if (selectedItem) {
        if(selectedItem->text().contains("File Dialog")) {
            QFileDialog dialog(this);
            dialog.setFileMode(QFileDialog::DirectoryOnly);
            if (dialog.exec()) {
                QStringList fileNames = dialog.selectedFiles();
                if(!fileNames[0].isEmpty()) {
                    caTextEntry::FormatType fType = getFormatType();
                    // convert the type
                    CaQtDM_Lib_Interface::formatsType fTypeNew;
                    if(fType == caTextEntry::octal) fTypeNew = CaQtDM_Lib_Interface::octal;
                    else if(fType == caTextEntry::hexadecimal) fTypeNew = CaQtDM_Lib_Interface::hexadecimal;
                    else if(fType == caTextEntry::string) fTypeNew = CaQtDM_Lib_Interface::string;
                    else fTypeNew = CaQtDM_Lib_Interface::decimal;
                    caDataInterface->TreatRequestedValue(getPV(), fileNames[0], fTypeNew, this);
                }
            }
        }
        else if(selectedItem->text().contains("Input Dialog")) {
            bool ok;
            QString text = QInputDialog::getText(this, tr("Input data"), tr("Input:"), QLineEdit::Normal,"", &ok);
            if (ok && !text.isEmpty()) {
                caTextEntry::FormatType fType = getFormatType();
                // convert the type
                CaQtDM_Lib_Interface::formatsType fTypeNew;
                if(fType == caTextEntry::octal) fTypeNew = CaQtDM_Lib_Interface::octal;
                else if(fType == caTextEntry::hexadecimal) fTypeNew = CaQtDM_Lib_Interface::hexadecimal;
                else if(fType == caTextEntry::string) fTypeNew = CaQtDM_Lib_Interface::string;
                else fTypeNew = CaQtDM_Lib_Interface::decimal;
                caDataInterface->TreatRequestedValue(getPV(), text, fTypeNew, this);
            }
        }
    }
}

void caTextEntry::getWidgetInfo(QString* pv, int& nbPV, int& limitsDefault, int& precMode, int& limitsMode,
                               int& Precision, char* colMode, double& limitsMax, double& limitsMin){
    Q_UNUSED(limitsDefault);
    Q_UNUSED(precMode);
    Q_UNUSED(limitsMode);
    Q_UNUSED(Precision);
    Q_UNUSED(colMode);
    Q_UNUSED(limitsMax);
    Q_UNUSED(limitsMin);

    pv[0] = getPV().trimmed();
    nbPV = 1;
}
